#include <stdio.h>
#include "controlPanel.h"
#include <pthread.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../simulatorNetworkInterface/simulatorNetworkInterface.h"
#include "../storage/storage.h"
#include <semaphore.h>
#include "../search/search.h"
#include "../datalog/datalog.h"

#define CMD_START 1
#define CMD_STATUS 2
#define CMD_EXIT 3
#define CMD_WRITE_STATE_TO_FILE 4
#define CMD_PROGRAM 5

int input = 0;
sem_t semCommandFinished;

void inputRemoveNewline(char *str, int size){
    fgets(str, size, stdin);
    for(int i = 0; i < strlen(str); i++){
        if(str[i] == '\n'){
            str[i] = '\0';
            return;
        }
    }
}

void *inputLoop(){
    char userInput[1000];
    while(1){
        sem_wait(&semCommandFinished);

        inputRemoveNewline(userInput, 1000);

        // Pause / start command
        if(strcmp(userInput, "start") == 0){
            input = CMD_START;
        }else if(strcmp(userInput, "status") == 0){
            input = CMD_STATUS;
        }else if(strcmp(userInput, "exit") == 0){
            input = CMD_EXIT;
        }else if(strcmp(userInput, "write state") == 0){
            input = CMD_WRITE_STATE_TO_FILE;
        }else if(strcmp(userInput, "program") == 0){
            input = CMD_PROGRAM;
        }else {
            printf("Invalid command\n");
            sem_post(&semCommandFinished);
        }
    }
    return NULL;
}

void cmdStart(int *simulationRunning){
    if(*simulationRunning){
        printf("Pausing\n");
    }else{
        printf("Starting\n");
    }
    *simulationRunning = !*simulationRunning;
}

void printStatusForCell(uint32_t cell){
    printf("\033[0;34m%d\033[0m [%d] {%f, %f} \t : \033[0;32m%d\033[0m => ", cell, (int)cellValue(cell), totalDerivative(cell), der3(cell) ,storageGetNmbOfDataPoints(cell));
    dataPoint *p;
    for(int i = 0; i < storageGetNmbOfDataPoints(cell); i++){
        p = storageGetDatapointNmbX(cell, i);
        printf("(%d, %d) ", p->data.time, p->data.meas);
    }
}

/**
 * A command which prints out all the currently monitored cells and their cell value
 */
void cmdStatus(){
    printf("\033[1;32mSTATUS:\033[0m\n"); // STATUS in green
    printf("\033[0;34m%s\033[0m [%s] {%s, %s} : %s \n", "cell", "cellValue", "td", "der3", "number of points");
    uint32_t cellList[MAX_NMB_OF_CELLS];
    uint16_t nmbOfCells = storageGetListOfCells(cellList, MAX_NMB_OF_CELLS);
    for(int i = 0; i < nmbOfCells; i++){
        printStatusForCell(cellList[i]);
        printf("\n");
    }
    printDebugDeleteInfo();
}

void cmdWriteState(){
    datalogWriteAllMonitoredCells();
}

void startOrResumeSim(){
    printf("Start or resume sim? [resume, start]\n");
    char in[100];
    inputRemoveNewline(in, 100);
    if(strcmp(in, "resume") == 0){
        printf("Please enter simnmb\n");
        inputRemoveNewline(in, 100);
        setSimNmb(atoi(in));
    }else{
        printf("Starting new sim\n");
        simulatorStart();
    }
    input = CMD_START;
}

void cmdProgram(){
    printf("Which bit would you like to reprogram?\n");
    char i[100];
    inputRemoveNewline(i, 100);
    simulatorProgram(atoi(i));
    printf("Reprogrammed %s\n", i);
}

void controlPanelStart(void){
//    storageInitNVMModel(5*5 + 1, 5, 5, 4); // Ample Storage
    storageInitNVMModel(15 + 1, 5, 3, 3); // Limited Storage

    // Start or resume sim?
    startOrResumeSim();
    printf("SIM STARTED\n");

    pthread_t t;
    pthread_create(&t, NULL, inputLoop, NULL);
    int simulationRunning = 0;

    while(1){
        switch(input){
            case CMD_START:
                cmdStart(&simulationRunning);
                break;
            case CMD_EXIT:
                exit(31);
            case CMD_STATUS:
                cmdStatus();
                break;
            case CMD_WRITE_STATE_TO_FILE:
                cmdWriteState();
                break;
            case CMD_PROGRAM:
                cmdProgram();
                break;
            default:
                if(simulationRunning){
                    searchSweep();
                    cmdWriteState();
                }
                continue;
        }
        input = 0;
        sem_post(&semCommandFinished);
    }
    storageFreeNVMModel();
    pthread_join(t, NULL);
}