#include "datalog.h"
#include <stdio.h>
#include <stdint.h>
#include "../storage/storage.h"
#include "../search/search.h"
#include <time.h>
#include <assert.h>

const char a[] = "log.csv";

int fileInitialized = 0;
char filenameCSV[100];
char filenamePrune[100];
char filenameDelete[100];


void initializeFile() {
    time_t t = time(NULL);
    struct tm tm = *localtime(&t);
    sprintf(filenameCSV, "%d-%d-%d %d:%d:%d.csv", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min,
            tm.tm_sec);
    sprintf(filenamePrune, "%d-%d-%d %d:%d:%d.prune", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min,
            tm.tm_sec);
    sprintf(filenameDelete, "%d-%d-%d %d:%d:%d.delete", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min,
            tm.tm_sec);
    FILE *csvFile = fopen(filenameCSV, "a");
    assert(csvFile != NULL);
    fprintf(csvFile, "iter, cell, time, value, cellV, td\n");
    fclose(csvFile);
    fileInitialized = 1;
}


int datalogWriteAllMonitoredCells(){
    if(!fileInitialized){
        initializeFile();
    }
    FILE *csvFile = fopen(filenameCSV, "a");
    if(csvFile == NULL){
        return 0;
    }

    uint32_t cellList[MAX_NMB_OF_CELLS];
    uint16_t nmbOfCells = storageGetListOfCells(cellList, MAX_NMB_OF_CELLS);
    uint8_t nmbOfPoints;
    for(int i = 0; i < nmbOfCells; i++){
        dataPoint *p;
        nmbOfPoints = storageGetNmbOfDataPoints(cellList[i]);
        for(int j = 0; j < nmbOfPoints; j++){
            p = storageGetDatapointNmbX(cellList[i], j);
            fprintf(csvFile, "%d, %d, %d, %d, %f, %f\n",
                    getCurrentTime(),
                    cellList[i],
                    p->data.time,
                    p->data.meas,
                    cellValue(cellList[i]),
                    totalDerivative(cellList[i]));
        }
    }

    fclose(csvFile);

    return 1;
}


int datalogWritePrunedCell(int cell, int typeOfDelete)
{
    FILE *pruneFile = fopen(filenamePrune, "a");
    if(pruneFile == NULL){
        return 0;
    }
    fprintf(pruneFile, "%d, %d, %d \n", getCurrentTime(), cell, typeOfDelete);
    fclose(pruneFile);
    return 1;
}

int datalogWriteDelete(int normal, int erratic, int same, int other)
{
    FILE *deleteFile = fopen(filenameDelete, "a");
    if(deleteFile == NULL){
        return 0;
    }
    fprintf(deleteFile, "%d, %d, %d, %d, %d \n", getCurrentTime(), normal, erratic, same, other);
    fclose(deleteFile);
    return 1;
}
