#include "gtest/gtest.h"
#include "gmock/gmock.h"
#include "search_pi.h"
#include "../storage/storage.h"
#include <iostream>

using namespace testing;
using namespace std;

class SearchTestSuite : public ::testing::Test {
protected:
    void SetUp() override {
        storageInitNVMModel(20, 5, 5, 5);
    }

    void TearDown() override {
        storageFreeNVMModel();
    }

    static void generateTestData() {
        uint16_t dataMeas[] = {1800, 1800, 1750, 1750, 1700, 1650, 1650, 1650, 1550, 1550, 1550, 1550, 1600, 1600};
        uint16_t dataTime[] = {10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120, 130, 140};
        dataPoint p;
        p.DATA = 0;
        for (int i = 0; i < 14; i++) {
            p.data.time = dataTime[i];
            p.data.meas = dataMeas[i];
            assert(storageAddDatapoint(&p, 23) == 1);
        }
    }

};

class SystemTestSuite : public ::testing::Test {
protected:
    void SetUp() override {
        storageInitNVMModel(20, 5, 5, 5);
    }

    void TearDown() override {
        storageFreeNVMModel();
    }

    static void generateTestData() {
        uint16_t dataMeas[] = {1800, 1800, 1750, 1750, 1700, 1650, 1650, 1650, 1550, 1550, 1550, 1550, 1600, 1600};
        uint16_t dataTime[] = {10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120, 130, 140};
        dataPoint p;
        p.DATA = 0;
        for (int i = 0; i < 14; i++) {
            p.data.time = dataTime[i];
            p.data.meas = dataMeas[i];
            assert(storageAddDatapoint(&p, 23) == 1);
        }
    }

    dataPoint list[20];

    void generatePointListDefault() {
        uint16_t value2[] = {1675, 1675, 1625, 1625, 1575};
        uint16_t value9[] = {1675, 1650, 1650, 1625, 1575};
        uint16_t value12[] = {1675, 1650, 1650, 1625, 1575};
        uint16_t value23[] = {1675, 1650, 1650, 1625, 1575};

        uint16_t time2[] = {20, 50, 80, 110, 140};
        uint16_t time9[] = {20, 50, 80, 110, 140};
        uint16_t time12[] = {20, 50, 80, 110, 140};
        uint16_t time23[] = {20, 50, 80, 110, 140};

        for (int i = 0; i < 5; i++) {
            dataPoint p2;
            p2.data.time = time2[i];
            p2.data.meas = value2[i];
            list[i*4] = p2;

            dataPoint p9;
            p9.data.time = time9[i];
            p9.data.meas = value9[i];
            list[i*4 + 1] = p9;

            dataPoint p12;
            p12.data.time = time12[i];
            p12.data.meas = value12[i];
            list[i*4 + 2] = p2;

            dataPoint p23;
            p23.data.time = time23[i];
            p23.data.meas = value23[i];
            list[i*4 + 3] = p23;
        }

    }

};

TEST_F(SearchTestSuite, PruneCell) {
    SearchTestSuite::generateTestData();

    // After first prune
    pruneCellSimple(23);
    dataPoint *pv = storageGetDatapointNmbX(23, 4);
    ASSERT_EQ(pv->data.next, 6);
    for (int i = 0; i < 3; i++) {
        pv = storageGetNextPtr(pv);
    }
    ASSERT_EQ(pv->data.next, 10);

    // Second Prune
    pruneCellSimple(23);
    pv = storageGetDatapointNmbX(23, 4);
    ASSERT_EQ(pv->data.time, 50);
    ASSERT_EQ(pv->data.next, 7);

    pv = storageGetDatapointNmbX(23, 8);
    ASSERT_EQ(pv->data.time, 120);
    ASSERT_EQ(pv->data.next, 13);

}

TEST_F(SearchTestSuite, Metrics) {
    SearchTestSuite::generateTestData();
    double d3 = der3(23);
    double td = totalDerivative(23);
    uint16_t abs = absoluteChange(23);
    ASSERT_DOUBLE_EQ(d3, 2.5);
    ASSERT_DOUBLE_EQ(td, -1.5384615384615385);
    ASSERT_EQ(abs, 200);
}

TEST_F(SearchTestSuite, CellValue) {
    SearchTestSuite::generateTestData();
    double cV = cellValue(23);
    ASSERT_DOUBLE_EQ(cV, 11.538461538461538);
}

TEST_F(SearchTestSuite, pointValueEval) {
    SearchTestSuite::generateTestData();
    dataPoint *p = storageGetDatapointNmbX(23, 2);
    ASSERT_EQ(datapointValue(p, 23, 0), 97);
}

// TODO: Fix test
/*
TEST_F(SystemTestSuite, considerPointNormal) {
    generatePointListDefault();
    uint32_t cells[4] = {2, 9, 12, 23};
    for(int i = 0; i < 20; i++){
        considerNewDatapoint(&list[i], cells[i % 4]);
    }

    // Add new point 170,1550 for cell 2, should delete t = 80
    dataPoint n;
    n.data.time = 170;
    n.data.meas = 1550;
    n.data.next = 0;
    considerNewDatapoint(&n, 2);

    // Check that t = 80 does not exist
    dataPoint *p = storageGetFirstDataPoint(2);
    while(p->data.next != 0){
        ASSERT_NE(p->data.time, 80);
        p = storageGetNextPtr(p);
    }

    // Adding a new point 200, 1800, should delete t = 140
    n.data.time = 200;
    n.data.meas = 1800;
    n.data.next = 0;
    considerNewDatapoint(&n, 2);

    // Check that t = 140 does not exist
    p = storageGetFirstDataPoint(2);
    while(p->data.next != 0){
        ASSERT_NE(p->data.time, 140);
        p = storageGetNextPtr(p);
    }

    // Adding a new point 230, 1800, should add and remove 200
    n.data.time = 230;
    n.data.meas = 1800;
    n.data.next = 0;
    considerNewDatapoint(&n, 2);

    // Check that t = 200 not does exist
    p = storageGetFirstDataPoint(2);
    while(p->data.next != 0){
        ASSERT_NE(p->data.time, 200);
        p = storageGetNextPtr(p);
    }

    // Adding a new point 300, 2000, should add and remove 50
    n.data.time = 300;
    n.data.meas = 2000;
    n.data.next = 0;
    considerNewDatapoint(&n, 2);

    // Check that t = 300 not does exist
    p = storageGetFirstDataPoint(2);
    while(p->data.next != 0){
        ASSERT_NE(p->data.time, 300);
        p = storageGetNextPtr(p);
    }

    // CELL 12
    // Addng new point 170, should delete t= 80
    n.data.time = 170;
    n.data.meas = 1550;
    n.data.next = 0;
    considerNewDatapoint(&n, 12);

    p = storageGetFirstDataPoint(12);
    while(p->data.next != 0){
        ASSERT_NE(p->data.time, 80);
        p = storageGetNextPtr(p);
    }
}
*/
TEST_F(SystemTestSuite, considerPointOverMax) {
    uint16_t value2[] = {1675, 1675, 1625, 1625, 1575, 1575};
    uint16_t value9[] = {1675, 1650, 1650, 1625, 1575};
    uint16_t value12[] = {1675, 1650, 1650, 1625, 1575};
    uint16_t value23[] = {1675, 1650, 1650, 1625};

    uint16_t time2[] = {20, 50, 80, 110, 140, 170};
    uint16_t time9[] = {20, 50, 80, 110, 140};
    uint16_t time12[] = {20, 50, 80, 110, 140};
    uint16_t time23[] = {20, 50, 80, 110};

    dataPoint n;

    // Cell 2
    for(int i = 0; i < 6; i++){
        n.data.time = time2[i];
        n.data.meas = value2[i];
        n.data.next = 0;
        considerNewDatapoint(&n, 2);
    }

    // 9 and 12
    for (int i = 0; i < 5; ++i) {
        n.data.time = time9[i];
        n.data.meas = value9[i];
        n.data.next = 0;
        considerNewDatapoint(&n, 9);


        n.data.time = time12[i];
        n.data.meas = value12[i];
        n.data.next = 0;
        considerNewDatapoint(&n, 12);
    }

    // 23
    for (int i = 0; i < 4; ++i) {
        n.data.time = time23[i];
        n.data.meas = value23[i];
        n.data.next = 0;
        considerNewDatapoint(&n, 23);
    }

    // Introducing new point to 23, should delete worst point in 2 which is t = 170
    n.data.time = 170;
    n.data.meas = 1600;
    n.data.next = 0;
    considerNewDatapoint(&n, 23);

    // Make sure t = 170 for cell 2 does not exist
    dataPoint *p = storageGetFirstDataPoint(2);
    while(p->data.next != 0){
        ASSERT_NE(p->data.time, 170);
        p = storageGetNextPtr(p);
    }

    // Make sure t = 170 exist for cell 23
    p = storageGetLatestDataPoint(23);
    ASSERT_EQ(p->data.time, 170);

}

TEST_F(SearchTestSuite, ReprogramEval) {
    SearchTestSuite::generateTestData();
    dataPoint *p = storageGetDatapointNmbX(23, 2);
    ASSERT_EQ(datapointValue(p, 23, 0), 97);
}