#ifndef PREVENTIONSYSTEM_SEARCH_H
#define PREVENTIONSYSTEM_SEARCH_H

#include <stdint.h>
#include "../storage/storage.h"

#define SEARCH_WATCH_VALUE 1550
#define SEARCH_ERROR_VALUE 1150
#define SEARCH_MIN_DELTA_T 24

#define REPROGRAM_CIRITCAL_ABSOLUTE (SEARCH_ERROR_VALUE)
#define REPROGRAM_WARNING_ABSOLUTE (SEARCH_ERROR_VALUE + 200)
#define REPROGRAM_CIRITCAL_TD (-0.2)
#define REPGORAM_ABS_VALUE_DELTA 100
#define REPGORAM_DELTA_POINT_BASE 2


#define C_TREND_LONG_WEIGHT 1
#define C_TREND_SHORT_WEIGHT 0
#define C_ABSOLUTE_WEIGHT 0.05

#define DP_DISTANCE_WEIGHT 1.0
#define DP_DELTA_WEIGHT 1.0
#define DP_DISTANCE_EDGE_WEIGHT 300

#define SIMPLE_PRUNE_LIMIT 2

#define CRITICAL_CELL_POINT_AMOUNT 3
#define CRITICAL_CELL_POINT_TIME 100

#define PRUNE_ERRATIC_DELTA_T_THRESHOLD 200
#define PRUNE_ERRATIC_TD_THRESHOLD (-0.05)

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Searches part of the flash with WatchRoutine()
 * @param startWord Word to start searching from
 * @param endWord Word to stop the search
 */
void searchPartOfFlash(uint16_t startWord, uint16_t endWord);

/**
 *
 * @param cell
 * @param icell
 */
void watchRoutine(uint32_t cell, uint16_t icell);

void analyzeCell(uint32_t cell);

uint8_t pruneCellSimple(uint32_t cell);

uint8_t pruneCellErratic(uint32_t cell);

double datapointValue(dataPoint *p, uint32_t cell, uint8_t isNewCell);

double cellValue(uint32_t cell);

uint16_t getCurrentTime();

void searchSweep();

double totalDerivative(uint32_t cell);

double der3(uint32_t cell);

void printDebugDeleteInfo();


#ifdef __cplusplus
}
#endif

#endif //PREVENTIONSYSTEM_SEARCH_H
