import numpy as np

class Config(object):
    def __init__(self):
        # TCP Server
        self.SERVER_ADDR = "localhost"
        self.SERVER_PORT = 8181

        # Simulation parameters
        self.FLASH_SIZE_KB = 128
        self.ICELL_STD = 0.9149
        self.ICELL_MEAN = 20.6835
        self.VERBOSE = False

        self.DELTA_I = 0.25
        self.P_P_CHANGE = 0.1
        self.P_N_CHANGE = self.P_P_CHANGE
        self.P_PN_INITIAL = 0.1

        # Error Mode Configuration
        # Moving bits
        self.SELECTED_RANDOM_BITS = [2, 54, 78, 33]
        #self.SELECTED_RANDOM_BITS = [13]
        self.MOVING_BIT_NOISE_STD = 0.1

        # Erratic bits
        self.NMB_OF_ERRATIC_BITS = 1
        #self.SELECTED_ERRATIC_BITS = [87]
        self.SELECTED_ERRATIC_BITS = []
        #self.ERRATIC_BIT_CHANCE = 0.01
        self.ERRATIC_BIT_CHANCE = 0.005

        self.HISTORY_BITS = self.SELECTED_RANDOM_BITS + self.SELECTED_ERRATIC_BITS

        self.INTERPOLATION_B = -0.000476
        self.INTERPOLATION_B_STD = 0.0001
        self.INTERPOLATION_C = 6
        self.INTERPOLATION_C_STD = 4

        # Mode configuration (only T/F)
        self.ONLY_FEW_CELLS_CHANGE = True
        self.REPEAT_CSV_FILES = False
        self.CSV_FILE_DIRECTORY = "./generatedCSV"
        self.NMB_BITS_TO_PROCESS = int(self.FLASH_SIZE_KB * 1024 * 8 / 16)

        # Plot configuration
        self.PLOT_MOVING_BITS = True

        if np.logical_and(self.ONLY_FEW_CELLS_CHANGE, self.REPEAT_CSV_FILES):
            raise ConfigErrorException("Incorrect mode config")


    def setModeToCSVRead(self):
        self.ONLY_FEW_CELLS_CHANGE = False
        self.REPEAT_CSV_FILES = False

    def setModeToOnlyFewCells(self):
        self.ONLY_FEW_CELLS_CHANGE = True
        self.REPEAT_CSV_FILES = False

    def setModeToAllCells(self):
        self.ONLY_FEW_CELLS_CHANGE = False
        self.REPEAT_CSV_FILES = False

ac = Config()

class ConfigErrorException(Exception):
    def __init__(self, message):
        super().__init__(message)



