import socketserver
from config import ac
from simulation import SimulationManager
from tqdm import tqdm
import json
import numpy as np

simManager = SimulationManager()

class connectionHandler(socketserver.BaseRequestHandler):
    @staticmethod
    def handleModeCmd(simNmb, mode):
        try:
            sim = simManager.activeSimulations[simNmb]
            if mode == "SIM_ALL":
                sim.ac.setModeToAllCells()
            elif mode == "SIM_SOME":
                sim.ac.setModeToOnlyFewCells()
            elif mode == "CSV_EXTRACT":
                sim.ac.setModeToCSVRead()
            else:
                return "error_unknown_mode\n"
            return "ok:" + str(simNmb) + "\n"
        except KeyError:
            return "error_invalid_sim_nmb\n"

    @staticmethod
    def handlePrgsTime(simNmb, n):
        try:
            sim = simManager.activeSimulations[simNmb]
            for i in tqdm(range(n)):
                sim.progressTime()
            response = {"status": "ok"}
        except KeyError:
            response = {"status": "error_invalid_sim_nbm"}
        return json.dumps(response)

    @staticmethod
    def handleRqstSbData(simNmb, bit):
        try:
            sim = simManager.activeSimulations[simNmb]
            response = {"status": "ok", "data": [sim[bit] * 100]}
        except KeyError:
            response = {"status": "error_invalid_sim_nbm"}
        return json.dumps(response)

    @staticmethod
    def handleRqstWordData(simNmb, word):
        try:
            sim = simManager.activeSimulations[simNmb]
            # Loop through all bits
            r = []
            # TODO: double check that the decode of word => cell is correct
            for i in range(32):
                r.append(sim[word * 32 + i] * 100)
            response = {"status": "ok", "data": r}
        except KeyError:
            response = {"status": "error_invalid_sim_nmb"}
        return json.dumps(response)

    @staticmethod
    def handleNewSim():
        simNmb = simManager.requestNewSimulation()
        response = {"status": "ok", "sim_nmb": simNmb}
        print(
            "Starting new sim = {}, moving bits are {}".format(simNmb, simManager.activeSimulations[simNmb].movingBits))
        return json.dumps(response)

    @staticmethod
    def handleRqstTrueMean(simNmb):
        try:
            sim = simManager.activeSimulations[simNmb]
            response = {"status": "ok", "data": sim.mean()}
        except KeyError:
            response = {"status": "error_invalid_sim_nmb"}
        return json.dumps(response)

    @staticmethod
    def handleRqstTime(simNmb):
        try:
            sim = simManager.activeSimulations[simNmb]
            response = {"status": "ok", "data": [sim.t]}
        except KeyError:
            response = {"status": "error_invalid_sim_nmb"}
        return json.dumps(response)

    @staticmethod
    def handleAddBitGraph(simNmb, bit):
        try:
            sim = simManager.activeSimulations[simNmb]
            sim.addHistoryBit(bit)
            response = {"status": "ok"}
        except KeyError:
            response = {"status": "error_invalid_sim_nmb"}
        return json.dumps(response)

    @staticmethod
    def sweep(simNmb, threshold):
        try:
            sim = simManager.activeSimulations[simNmb]
            r = sim.sweep(threshold / 100)
            response = {"status": "ok", "data": r}
        except KeyError:
            response = {"status": "error_invalid_sim_nmb"}
        return json.dumps(response)

    @staticmethod
    def program(simNmb, bit):
        try:
            sim = simManager.activeSimulations[simNmb]
            sim.program(bit)
            response = {"status": "ok"}
        except KeyError:
            response = {"status": "error_invalid_sim_nmb"}
        return json.dumps(response)

    def handle(self):
        msg = json.loads(self.request.recv(4096).strip().decode("utf-8"))
        response = json.dumps({"status": "error_invalid_cmd"})
        if msg["cmd"] == "start_sim":
            response = self.handleNewSim()
        elif msg["cmd"] == "set_mode":
            response = self.handleModeCmd(int(msg["sim_nmb"]), msg["mode"])
        elif msg["cmd"] == "prgs_time":
            response = self.handlePrgsTime(int(msg["sim_nmb"]), int(msg["n"]))
        elif msg["cmd"] == "rqst_data_sb":
            response = self.handleRqstSbData(int(msg["sim_nmb"]), int(msg["bit"]))
        elif msg["cmd"] == "rqst_data_word":
            response = self.handleRqstWordData(int(msg["sim_nmb"]), int(msg["word"]))
        elif msg["cmd"] == "rqst_true_mean":
            response = self.handleRqstTrueMean(int(msg["sim_nmb"]))
        elif msg["cmd"] == "rqst_time":
            response = self.handleRqstTime(int(msg["sim_nmb"]))
        elif msg["cmd"] == "add_bit_graph":
            response = self.handleAddBitGraph(int(msg["sim_nmb"]), int(msg["bit"]))
        elif msg["cmd"] == "sweep":
            response = self.sweep(int(msg["sim_nmb"]), int(msg["threshold"]))
        elif msg["cmd"] == "program":
            response = self.program(int(msg["sim_nmb"]), int(msg["bit"]))
        self.request.sendall(bytearray(response, "utf-8"))


with socketserver.TCPServer((ac.SERVER_ADDR, ac.SERVER_PORT), connectionHandler) as tcpServer:
    tcpServer.serve_forever()