import unittest
import numpy as np
from simulation import Simulation, SimulationManager
from matplotlib import pyplot
import scipy.stats as stats
from tqdm import tqdm

PLOTS = False

class testSimulator(unittest.TestCase):
    def setUp(self):
        self.simulator = Simulation()

    def test_fiftyPercentIsLowCurrent(self):
        self.simulator.ac.setModeToAllCells()
        unique, counts = np.unique(self.simulator.values, return_counts=True)
        self.assertEqual(int((self.simulator.ac.FLASH_SIZE_KB * 1024 * 8) / 2), counts[0])

    def test_highCurrentGaussianDistributed(self):
        self.simulator.ac.setModeToAllCells()
        highCurrentCells = self.simulator.values[np.where(self.simulator.values > 0.5)]
        pValue = stats.normaltest(highCurrentCells)[1]
        self.assertLess(pValue, 2)

    def test_errorsCorrectlyInitialized(self):
        self.simulator.ac.setModeToAllCells()
        for i, v in enumerate(self.simulator.values[self.simulator.movingBits]):
            self.assertNotEqual(v, 0.5)

    def testProgressionOfTime(self):
        self.simulator.ac.setModeToOnlyFewCells()
        self.simulator.ac.NMB_BITS_TO_PROCESS = int(self.simulator.ac.FLASH_SIZE_KB * 1024 * 8 / 64)
        self.simulator.progressTime()

    def testSingleCell(self):
        if not PLOTS:
            self.assertTrue(True)
            return
        bit = 2
        y = [self.simulator.values[bit]]
        for i in tqdm(range(1, 150)):
            self.simulator.progressTime()
            y.append(self.simulator.values[bit])

        pyplot.plot(y)
        pyplot.grid(True)
        #pyplot.ylim((16, 26))
        pyplot.show()
        self.simulator.plot.plot()


class testSimulationManager(unittest.TestCase):
    def setUp(self):
        self.manager = SimulationManager()

    def test_multipleSimulationsCanBeStarted(self):
        activeIds = []
        for i in range(0, 3):
            activeIds.append(self.manager.requestNewSimulation())

        self.assertEqual(len(activeIds), len(set(activeIds)))


if __name__ == '__main__':
    unittest.main()
